@@ -32,6 +32,9 @@ gchar *mm_common_build_capabilities_string (const MMModemCapability *capabilitie
 gchar *mm_common_build_bands_string (const MMModemBand *bands,
                                      guint n_bands);
 
+gchar *mm_common_build_ports_string (const MMModemPortInfo *ports,
+                                     guint n_ports);
+
 gchar *mm_common_build_sms_storages_string (const MMSmsStorage *storages,
                                             guint n_storages);
 
@@ -56,6 +59,21 @@ MMBearerAllowedAuth   mm_common_get_allowed_auth_from_string (const gchar *str,
                                                               GError **error);
 MMSmsStorage          mm_common_get_sms_storage_from_string  (const gchar *str,
                                                               GError **error);
+MMSmsCdmaTeleserviceId   mm_common_get_sms_cdma_teleservice_id_from_string   (const gchar *str,
+                                                                              GError **error);
+MMSmsCdmaServiceCategory mm_common_get_sms_cdma_service_category_from_string (const gchar *str,
+                                                                              GError **error);
+MMOmaFeature          mm_common_get_oma_features_from_string (const gchar *str,
+                                                              GError **error);
+MMOmaSessionType      mm_common_get_oma_session_type_from_string (const gchar *str,
+                                                                  GError **error);
+
+GArray          *mm_common_ports_variant_to_garray (GVariant *variant);
+MMModemPortInfo *mm_common_ports_variant_to_array  (GVariant *variant,
+                                                 guint *n_ports);
+GVariant        *mm_common_ports_array_to_variant  (const MMModemPortInfo *ports,
+                                                    guint n_ports);
+GVariant        *mm_common_ports_garray_to_variant (GArray *array);
 
 GArray       *mm_common_sms_storages_variant_to_garray (GVariant *variant);
 MMSmsStorage *mm_common_sms_storages_variant_to_array  (GVariant *variant,
@@ -93,6 +111,14 @@ GVariant          *mm_common_capability_combinations_garray_to_variant (GArray *
 GVariant          *mm_common_build_capability_combinations_any         (void);
 GVariant          *mm_common_build_capability_combinations_none        (void);
 
+GArray                              *mm_common_oma_pending_network_initiated_sessions_variant_to_garray (GVariant *variant);
+MMOmaPendingNetworkInitiatedSession *mm_common_oma_pending_network_initiated_sessions_variant_to_array  (GVariant *variant,
+                                                                                                         guint *n_modes);
+GVariant                            *mm_common_oma_pending_network_initiated_sessions_array_to_variant  (const MMOmaPendingNetworkInitiatedSession *modes,
+                                                                                                         guint n_modes);
+GVariant                            *mm_common_oma_pending_network_initiated_sessions_garray_to_variant (GArray *array);
+GVariant                            *mm_common_build_oma_pending_network_initiated_sessions_default     (void);
+
 typedef gboolean (*MMParseKeyValueForeachFn) (const gchar *key,
                                               const gchar *value,
                                               gpointer user_data);
