@@ -27,8 +27,9 @@
 #include "mm-error-helpers.h"
 #include "mm-log.h"
 #include "mm-sim-mbim.h"
+#include "mm-modem-helpers-mbim.h"
 
-G_DEFINE_TYPE (MMSimMbim, mm_sim_mbim, MM_TYPE_SIM)
+G_DEFINE_TYPE (MMSimMbim, mm_sim_mbim, MM_TYPE_BASE_SIM)
 
 /*****************************************************************************/
 
@@ -39,10 +40,10 @@ peek_device (gpointer self,
              gpointer user_data)
 {
     MMBaseModem *modem = NULL;
-    MMMbimPort *port;
+    MMPortMbim *port;
 
     g_object_get (G_OBJECT (self),
-                  MM_SIM_MODEM, &modem,
+                  MM_BASE_SIM_MODEM, &modem,
                   NULL);
     g_assert (MM_IS_BASE_MODEM (modem));
 
@@ -59,7 +60,7 @@ peek_device (gpointer self,
         return FALSE;
     }
 
-    *o_device = mm_mbim_port_peek_device (port);
+    *o_device = mm_port_mbim_peek_device (port);
     return TRUE;
 }
 
@@ -67,13 +68,13 @@ peek_device (gpointer self,
 /* Load SIM identifier */
 
 static gchar *
-load_sim_identifier_finish (MMSim *self,
+load_sim_identifier_finish (MMBaseSim *self,
                             GAsyncResult *res,
                             GError **error)
 {
     if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error))
         return NULL;
-    return (gchar *)g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (res));
+    return g_strdup ((gchar *)g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (res)));
 }
 
 static void
@@ -87,7 +88,7 @@ simid_subscriber_ready_state_ready (MbimDevice *device,
 
     response = mbim_device_command_finish (device, res, &error);
     if (response &&
-        mbim_message_command_done_get_result (response, &error) &&
+        mm_mbim_helper_response_get_result (response, &error) &&
         mbim_message_subscriber_ready_status_response_parse (
             response,
             NULL, /* ready_state */
@@ -97,7 +98,7 @@ simid_subscriber_ready_state_ready (MbimDevice *device,
             NULL, /* telephone_numbers_count */
             NULL, /* telephone_numbers */
             &error))
-        g_simple_async_result_set_op_res_gpointer (simple, sim_iccid, NULL);
+        g_simple_async_result_set_op_res_gpointer (simple, sim_iccid, (GDestroyNotify)g_free);
     else
         g_simple_async_result_take_error (simple, error);
 
@@ -108,7 +109,7 @@ simid_subscriber_ready_state_ready (MbimDevice *device,
 }
 
 static void
-load_sim_identifier (MMSim *self,
+load_sim_identifier (MMBaseSim *self,
                      GAsyncReadyCallback callback,
                      gpointer user_data)
 {
@@ -135,13 +136,13 @@ load_sim_identifier (MMSim *self,
 /* Load IMSI */
 
 static gchar *
-load_imsi_finish (MMSim *self,
+load_imsi_finish (MMBaseSim *self,
                             GAsyncResult *res,
                             GError **error)
 {
     if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error))
         return NULL;
-    return (gchar *)g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (res));
+    return g_strdup ((gchar *)g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (res)));
 }
 
 static void
@@ -155,7 +156,7 @@ imsi_subscriber_ready_state_ready (MbimDevice *device,
 
     response = mbim_device_command_finish (device, res, &error);
     if (response &&
-        mbim_message_command_done_get_result (response, &error) &&
+        mm_mbim_helper_response_get_result (response, &error) &&
         mbim_message_subscriber_ready_status_response_parse (
             response,
             NULL, /* ready_state */
@@ -165,7 +166,7 @@ imsi_subscriber_ready_state_ready (MbimDevice *device,
             NULL, /* telephone_numbers_count */
             NULL, /* telephone_numbers */
             &error))
-        g_simple_async_result_set_op_res_gpointer (simple, subscriber_id, NULL);
+        g_simple_async_result_set_op_res_gpointer (simple, subscriber_id, (GDestroyNotify)g_free);
     else
         g_simple_async_result_take_error (simple, error);
 
@@ -176,7 +177,7 @@ imsi_subscriber_ready_state_ready (MbimDevice *device,
 }
 
 static void
-load_imsi (MMSim *self,
+load_imsi (MMBaseSim *self,
            GAsyncReadyCallback callback,
            gpointer user_data)
 {
@@ -199,11 +200,145 @@ load_imsi (MMSim *self,
     mbim_message_unref (message);
 }
 
+/*****************************************************************************/
+/* Load operator identifier */
+
+static gchar *
+load_operator_identifier_finish (MMBaseSim *self,
+                                 GAsyncResult *res,
+                                 GError **error)
+{
+    MbimProvider *provider;
+
+    if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error))
+        return NULL;
+
+    provider = (MbimProvider *)g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (res));
+    return g_strdup (provider->provider_id);
+}
+
+static void
+load_operator_identifier_ready (MbimDevice *device,
+                                GAsyncResult *res,
+                                GSimpleAsyncResult *simple)
+{
+    MbimMessage *response;
+    GError *error = NULL;
+    MbimProvider *provider;
+
+    response = mbim_device_command_finish (device, res, &error);
+    if (response &&
+        mm_mbim_helper_response_get_result (response, &error) &&
+        mbim_message_home_provider_response_parse (
+            response,
+            &provider,
+            &error))
+        g_simple_async_result_set_op_res_gpointer (simple, provider, (GDestroyNotify)mbim_provider_free);
+    else
+        g_simple_async_result_take_error (simple, error);
+
+    if (response)
+        mbim_message_unref (response);
+    g_simple_async_result_complete (simple);
+    g_object_unref (simple);
+}
+
+static void
+load_operator_identifier (MMBaseSim *self,
+                          GAsyncReadyCallback callback,
+                          gpointer user_data)
+{
+    MbimDevice *device;
+    MbimMessage *message;
+    GSimpleAsyncResult *result;
+
+    if (!peek_device (self, &device, callback, user_data))
+        return;
+
+    result = g_simple_async_result_new (G_OBJECT (self), callback, user_data, load_operator_identifier);
+
+    message = mbim_message_home_provider_query_new (NULL);
+    mbim_device_command (device,
+                         message,
+                         10,
+                         NULL,
+                         (GAsyncReadyCallback)load_operator_identifier_ready,
+                         result);
+    mbim_message_unref (message);
+}
+
+/*****************************************************************************/
+/* Load operator name */
+
+static gchar *
+load_operator_name_finish (MMBaseSim *self,
+                           GAsyncResult *res,
+                           GError **error)
+{
+    MbimProvider *provider;
+
+    if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error))
+        return NULL;
+
+    provider = (MbimProvider *)g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (res));
+    return g_strdup (provider->provider_name);
+}
+
+static void
+load_operator_name_ready (MbimDevice *device,
+                          GAsyncResult *res,
+                          GSimpleAsyncResult *simple)
+{
+    MbimMessage *response;
+    GError *error = NULL;
+    MbimProvider *provider;
+
+    response = mbim_device_command_finish (device, res, &error);
+    if (response &&
+        mm_mbim_helper_response_get_result (response, &error) &&
+        mbim_message_home_provider_response_parse (
+            response,
+            &provider,
+            &error))
+        g_simple_async_result_set_op_res_gpointer (simple, provider, (GDestroyNotify)mbim_provider_free);
+    else
+        g_simple_async_result_take_error (simple, error);
+
+    if (response)
+        mbim_message_unref (response);
+    g_simple_async_result_complete (simple);
+    g_object_unref (simple);
+}
+
+static void
+load_operator_name (MMBaseSim *self,
+                    GAsyncReadyCallback callback,
+                    gpointer user_data)
+{
+    MbimDevice *device;
+    MbimMessage *message;
+    GSimpleAsyncResult *result;
+
+    if (!peek_device (self, &device, callback, user_data))
+        return;
+
+    result = g_simple_async_result_new (G_OBJECT (self), callback, user_data, load_operator_name);
+
+    message = mbim_message_home_provider_query_new (NULL);
+    mbim_device_command (device,
+                         message,
+                         10,
+                         NULL,
+                         (GAsyncReadyCallback)load_operator_name_ready,
+                         result);
+    mbim_message_unref (message);
+}
+
 /*****************************************************************************/
 /* Send PIN */
 
 static gboolean
-send_pin_finish (MMSim *self,
+send_pin_finish (MMBaseSim *self,
                  GAsyncResult *res,
                  GError **error)
 {
@@ -222,7 +357,7 @@ pin_set_enter_ready (MbimDevice *device,
 
     response = mbim_device_command_finish (device, res, &error);
     if (response &&
-        !mbim_message_command_done_get_result (response, &error)) {
+        !mm_mbim_helper_response_get_result (response, &error)) {
         /* Sending PIN failed, build a better error to report */
         if (mbim_message_pin_response_parse (
                 response,
@@ -252,7 +387,7 @@ pin_set_enter_ready (MbimDevice *device,
 }
 
 static void
-send_pin (MMSim *self,
+send_pin (MMBaseSim *self,
           const gchar *pin,
           GAsyncReadyCallback callback,
           gpointer user_data)
@@ -294,7 +429,7 @@ send_pin (MMSim *self,
 /* Send PUK */
 
 static gboolean
-send_puk_finish (MMSim *self,
+send_puk_finish (MMBaseSim *self,
                  GAsyncResult *res,
                  GError **error)
 {
@@ -314,7 +449,7 @@ puk_set_enter_ready (MbimDevice *device,
 
     response = mbim_device_command_finish (device, res, &error);
     if (response &&
-        !mbim_message_command_done_get_result (response, &error)) {
+        !mm_mbim_helper_response_get_result (response, &error)) {
         /* Sending PUK failed, build a better error to report */
         if (mbim_message_pin_response_parse (
                 response,
@@ -344,7 +479,7 @@ puk_set_enter_ready (MbimDevice *device,
 }
 
 static void
-send_puk (MMSim *self,
+send_puk (MMBaseSim *self,
           const gchar *puk,
           const gchar *new_pin,
           GAsyncReadyCallback callback,
@@ -387,7 +522,7 @@ send_puk (MMSim *self,
 /* Enable PIN */
 
 static gboolean
-enable_pin_finish (MMSim *self,
+enable_pin_finish (MMBaseSim *self,
                    GAsyncResult *res,
                    GError **error)
 {
@@ -404,7 +539,7 @@ pin_set_enable_ready (MbimDevice *device,
 
     response = mbim_device_command_finish (device, res, &error);
     if (response) {
-        mbim_message_command_done_get_result (response, &error);
+        mm_mbim_helper_response_get_result (response, &error);
         mbim_message_unref (response);
     }
 
@@ -424,7 +559,7 @@ pin_set_enable_ready (MbimDevice *device,
 }
 
 static void
-enable_pin (MMSim *self,
+enable_pin (MMBaseSim *self,
             const gchar *pin,
             gboolean enabled,
             GAsyncReadyCallback callback,
@@ -467,7 +602,7 @@ enable_pin (MMSim *self,
 /* Change PIN */
 
 static gboolean
-change_pin_finish (MMSim *self,
+change_pin_finish (MMBaseSim *self,
                    GAsyncResult *res,
                    GError **error)
 {
@@ -484,7 +619,7 @@ pin_set_change_ready (MbimDevice *device,
 
     response = mbim_device_command_finish (device, res, &error);
     if (response) {
-        mbim_message_command_done_get_result (response, &error);
+        mm_mbim_helper_response_get_result (response, &error);
         mbim_message_unref (response);
     }
 
@@ -504,7 +639,7 @@ pin_set_change_ready (MbimDevice *device,
 }
 
 static void
-change_pin (MMSim *self,
+change_pin (MMBaseSim *self,
             const gchar *old_pin,
             const gchar *new_pin,
             GAsyncReadyCallback callback,
@@ -545,7 +680,7 @@ change_pin (MMSim *self,
 
 /*****************************************************************************/
 
-MMSim *
+MMBaseSim *
 mm_sim_mbim_new_finish (GAsyncResult  *res,
                         GError       **error)
 {
@@ -560,9 +695,9 @@ mm_sim_mbim_new_finish (GAsyncResult  *res,
         return NULL;
 
     /* Only export valid SIMs */
-    mm_sim_export (MM_SIM (sim));
+    mm_base_sim_export (MM_BASE_SIM (sim));
 
-    return MM_SIM (sim);
+    return MM_BASE_SIM (sim);
 }
 
 void
@@ -576,7 +711,7 @@ mm_sim_mbim_new (MMBaseModem *modem,
                                 cancellable,
                                 callback,
                                 user_data,
-                                MM_SIM_MODEM, modem,
+                                MM_BASE_SIM_MODEM, modem,
                                 NULL);
 }
 
@@ -588,24 +723,22 @@ mm_sim_mbim_init (MMSimMbim *self)
 static void
 mm_sim_mbim_class_init (MMSimMbimClass *klass)
 {
-    MMSimClass *sim_class = MM_SIM_CLASS (klass);
+    MMBaseSimClass *base_sim_class = MM_BASE_SIM_CLASS (klass);
 
-    sim_class->load_sim_identifier = load_sim_identifier;
-    sim_class->load_sim_identifier_finish = load_sim_identifier_finish;
-    sim_class->load_imsi = load_imsi;
-    sim_class->load_imsi_finish = load_imsi_finish;
-    sim_class->send_pin = send_pin;
-    sim_class->send_pin_finish = send_pin_finish;
-    sim_class->send_puk = send_puk;
-    sim_class->send_puk_finish = send_puk_finish;
-    sim_class->enable_pin = enable_pin;
-    sim_class->enable_pin_finish = enable_pin_finish;
-    sim_class->change_pin = change_pin;
-    sim_class->change_pin_finish = change_pin_finish;
-
-    /* TODO: use MBIM_CID_HOME_PROVIDER */
-    sim_class->load_operator_identifier = NULL;
-    sim_class->load_operator_identifier_finish = NULL;
-    sim_class->load_operator_name = NULL;
-    sim_class->load_operator_name_finish = NULL;
+    base_sim_class->load_sim_identifier = load_sim_identifier;
+    base_sim_class->load_sim_identifier_finish = load_sim_identifier_finish;
+    base_sim_class->load_imsi = load_imsi;
+    base_sim_class->load_imsi_finish = load_imsi_finish;
+    base_sim_class->load_operator_identifier = load_operator_identifier;
+    base_sim_class->load_operator_identifier_finish = load_operator_identifier_finish;
+    base_sim_class->load_operator_name = load_operator_name;
+    base_sim_class->load_operator_name_finish = load_operator_name_finish;
+    base_sim_class->send_pin = send_pin;
+    base_sim_class->send_pin_finish = send_pin_finish;
+    base_sim_class->send_puk = send_puk;
+    base_sim_class->send_puk_finish = send_puk_finish;
+    base_sim_class->enable_pin = enable_pin;
+    base_sim_class->enable_pin_finish = enable_pin_finish;
+    base_sim_class->change_pin = change_pin;
+    base_sim_class->change_pin_finish = change_pin_finish;
 }
